==> brumbrumrally_0.6.orig.tar.gz: brumbrumrally-0.6/src/TrackState.cpp <==
/*
 * Copyright 2014 Peter Olsson
 * 
 * This file is part of Brum Brum Rally.
 * 
 * Brum Brum Rally is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Brum Brum Rally is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "TrackState.h"
#include "Track.h"
#include "StateManager.h"
#include <algorithm>
#include <cmath>
#include <limits>

TrackState::TrackState()
:	cars()
{
}

TrackState::~TrackState()
{
	delete[] cars;
}

void TrackState::setCars(int nCars)
{
	delete[] cars;
	cars = new TrackCarState[nCars];
}

void TrackState::removeCar(int carId, int nCars)
{
	std::copy(cars + carId + 1, cars + nCars, cars + carId);
}

void writeD16(NetworkMessage& msg, double min, double max, double value)
{
	int data = std::numeric_limits<Uint16>::max() * (value - min) / (max - min);
	if (data < 0)
	{
		data = 0;
	}
	else if (data > std::numeric_limits<Uint16>::max())
	{
		data = std::numeric_limits<Uint16>::max();
	}
	msg.write16(data);
}

void readD16(NetworkMessage& msg, double min, double max, double& value)
{
	value = msg.read16() * (max - min) / std::numeric_limits<Uint16>::max() + min; 
}

double wrapAngle(double angle)
{
    return angle - PI_TWO * std::floor(angle / PI_TWO);
}

void TrackState::write(NetworkMessage& msg, int nCars) const
{
	msg.write32(1000 * time);
	msg.write8(timeState);
	for (int i = 0; i < nCars; ++i)
	{
		const TrackCarState& car = cars[i];
		
		writeD16(msg, -CAR_SPEED_MAX, CAR_SPEED_MAX, car.velocity.x);
		writeD16(msg, -CAR_SPEED_MAX, CAR_SPEED_MAX, car.velocity.y);
		
		writeD16(msg, CAR_RADIUS, GAME_WIDTH  - CAR_RADIUS, car.position.x);
		writeD16(msg, CAR_RADIUS, GAME_HEIGHT - CAR_RADIUS, car.position.y);
		
		writeD16(msg, -50, 50, car.rotationVelocity);
		writeD16(msg, 0, PI_TWO, wrapAngle(car.rotation));
		
		msg.write8(car.tileOrderNum);
		msg.write8(car.lap);
		msg.write8(car.posFinished);
		
		msg.write32(1000 * car.timeFinished);
	}
}

void TrackState::read(NetworkMessage& msg, int nCars)
{
	time = 0.001 * msg.read32();
	timeState = msg.read8();
	for (int i = 0; i < nCars; ++i)
	{
		TrackCarState& car = cars[i];
		
		readD16(msg, -CAR_SPEED_MAX, CAR_SPEED_MAX, car.velocity.x);
		readD16(msg, -CAR_SPEED_MAX, CAR_SPEED_MAX, car.velocity.y);
		
		readD16(msg, CAR_RADIUS, GAME_WIDTH  - CAR_RADIUS, car.position.x);
		readD16(msg, CAR_RADIUS, GAME_HEIGHT - CAR_RADIUS, car.position.y);
		
		
		readD16(msg, -50, 50, car.rotationVelocity);
		readD16(msg, 0, PI_TWO, car.rotation);
		
		// When the cars are standing in start position
		// it is important that the rotation is precise 
		// to avoid skew horizontal orientaion.
		if (std::abs(car.rotation - PI_HALF) < 0.0001)
		{
			car.rotation = PI_HALF;
		}
		else if (std::abs(car.rotation - 3 * PI_HALF) < 0.0001)
		{
			car.rotation = -PI_HALF;
		}
		
		car.tileOrderNum = msg.read8();
		car.lap = msg.read8();
		car.posFinished = msg.read8();
		
		car.timeFinished = 0.001 * msg.read32();
	}
}

==> brumbrumrally-0.7.tar.gz: brumbrumrally-0.7/src/TrackState.cpp <==
/*
 * Copyright 2014 Peter Olsson
 * 
 * This file is part of Brum Brum Rally.
 * 
 * Brum Brum Rally is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Brum Brum Rally is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "TrackState.h"
#include "Track.h"
#include "StateManager.h"
#include <algorithm>
#include <cmath>
#include <limits>

TrackState::TrackState()
:	cars()
{
}

TrackState::~TrackState()
{
	delete[] cars;
}

void TrackState::setCars(int nCars)
{
	delete[] cars;
	cars = new TrackCarState[nCars];
}

void TrackState::removeCar(int carId, int nCars)
{
	std::copy(cars + carId + 1, cars + nCars, cars + carId);
}

void writeD16(NetworkMessage& msg, double min, double max, double value)
{
	int data = std::numeric_limits<Uint16>::max() * (value - min) / (max - min);
	if (data < 0)
	{
		data = 0;
	}
	else if (data > std::numeric_limits<Uint16>::max())
	{
		data = std::numeric_limits<Uint16>::max();
	}
	msg.write16(data);
}

void readD16(NetworkMessage& msg, double min, double max, double& value)
{
	value = msg.read16() * (max - min) / std::numeric_limits<Uint16>::max() + min; 
}

double wrapAngle(double angle)
{
    return angle - PI_TWO * std::floor(angle / PI_TWO);
}

void TrackState::write(NetworkMessage& msg, int nCars) const
{
	msg.write32(1000 * time);
	msg.write8(timeState);
	for (int i = 0; i < nCars; ++i)
	{
		const TrackCarState& car = cars[i];
		
		writeD16(msg, -CAR_SPEED_MAX, CAR_SPEED_MAX, car.velocity.x);
		writeD16(msg, -CAR_SPEED_MAX, CAR_SPEED_MAX, car.velocity.y);
		
		writeD16(msg, CAR_RADIUS, GAME_WIDTH  - CAR_RADIUS, car.position.x);
		writeD16(msg, CAR_RADIUS, GAME_HEIGHT - CAR_RADIUS, car.position.y);
		
		writeD16(msg, -50, 50, car.rotationVelocity);
		writeD16(msg, 0, PI_TWO, wrapAngle(car.rotation));
		
		msg.write8(car.tileOrderNum);
		msg.write8(car.lap);
		msg.write8(car.posFinished);
		
		msg.write32(1000 * car.timeFinished);
	}
}

void TrackState::read(NetworkMessage& msg, int nCars)
{
	time = 0.001 * msg.read32();
	timeState = msg.read8();
	for (int i = 0; i < nCars; ++i)
	{
		TrackCarState& car = cars[i];
		
		readD16(msg, -CAR_SPEED_MAX, CAR_SPEED_MAX, car.velocity.x);
		readD16(msg, -CAR_SPEED_MAX, CAR_SPEED_MAX, car.velocity.y);
		
		readD16(msg, CAR_RADIUS, GAME_WIDTH  - CAR_RADIUS, car.position.x);
		readD16(msg, CAR_RADIUS, GAME_HEIGHT - CAR_RADIUS, car.position.y);
		
		
		readD16(msg, -50, 50, car.rotationVelocity);
		readD16(msg, 0, PI_TWO, car.rotation);
		
		// When the cars are standing in start position
		// it is important that the rotation is precise 
		// to avoid skew horizontal orientation.
		if (std::abs(car.rotation - PI_HALF) < 0.0001)
		{
			car.rotation = PI_HALF;
		}
		else if (std::abs(car.rotation - 3 * PI_HALF) < 0.0001)
		{
			car.rotation = -PI_HALF;
		}
		
		car.tileOrderNum = msg.read8();
		car.lap = msg.read8();
		car.posFinished = msg.read8();
		
		car.timeFinished = 0.001 * msg.read32();
	}
}
